//
//  AtomClient.h
//  Atom
//
//  Created by Jonathan Younger on Wed May 05 2004.
//  Copyright (c) 2004 Daikini Software.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.

//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.

//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#import <Foundation/Foundation.h>
#import "Atom.h"
#import "ScopedElement.h"
#import "DcElement.h"

// Atom Client Notifications
extern NSString *AtomClientDidCreateAtomEntryNotification;
extern NSString *AtomClientDidUpdateAtomEntryNotification;
extern NSString *AtomClientDidDeleteAtomEntryNotification;
extern NSString *AtomClientDidRetrieveAtomEntryNotification;
extern NSString *AtomClientDidRetrieveAtomFeedNotification;
extern NSString *AtomClientRequestFailedNotification;

enum AtomClientOperation {
	AtomClientNoOperation,
	AtomClientCreatingAtomEntry,
	AtomClientUpdatingAtomEntry,
	AtomClientDeletingAtomEntry,
	AtomClientRetrievingAtomEntry,
	AtomClientRetrievingAtomFeed
};

@interface AtomClient : NSObject {
	NSString *userName;
	NSString *password;
	BOOL useSoap;
	NSString *userAgent;
	
	NSURLConnection *connection;
	NSMutableData *data;
	NSURLResponse *response;
	id delegate;
	int currentOperation;
	AtomEntry *entry;
}

- (id)initWithUserName:(NSString *)aUserName password:(NSString *)aPassword;

- (void)createAtomEntry:(AtomEntry *)anAtomEntry URL:(NSURL *)aURL delegate:(id)aDelegate;
- (void)updateAtomEntry:(AtomEntry *)anAtomEntry delegate:(id)aDelegate;
- (void)deleteAtomEntry:(AtomEntry *)anAtomEntry delegate:(id)aDelegate;
- (void)retrieveAtomEntryWithContentsOfURL:(NSURL *)aURL delegate:(id)aDelegate;
- (void)retrieveAtomFeedWithContentsOfURL:(NSURL *)aURL delegate:(id)aDelegate;

- (AtomEntry *)createAtomEntry:(AtomEntry *)anAtomEntry URL:(NSURL *)aURL returningError:(NSError **)anError;
- (AtomEntry *)updateAtomEntry:(AtomEntry *)anAtomEntry returningError:(NSError **)anError;
- (BOOL)deleteAtomEntry:(AtomEntry *)anAtomEntry returningError:(NSError **)anError;
- (AtomEntry *)retrieveAtomEntryWithContentsOfURL:(NSURL *)aURL returningError:(NSError **)anError;
- (AtomFeed *)retrieveAtomFeedWithContentsOfURL:(NSURL *)aURL returningError:(NSError **)anError;


- (void)cancel;

- (NSString *)userName;
- (void)setUserName:(NSString *)aUserName;

- (NSString *)password;
- (void)setPassword:(NSString *)aPassword;

- (BOOL)useSoap;
- (void)setUseSoap:(BOOL)flag;

- (NSString *)userAgent;
- (void)setUserAgent:(NSString *)aUserAgent;

@end

// Delegate Methods
@interface NSObject (AtomClientDelegate)
- (void)client:(AtomClient *)aClient didCreateAtomEntry:(AtomEntry *)anAtomEntry;
- (void)client:(AtomClient *)aClient didUpdateAtomEntry:(AtomEntry *)anAtomEntry;
- (void)client:(AtomClient *)aClient didDeleteAtomEntry:(AtomEntry *)anAtomEntry;
- (void)client:(AtomClient *)aClient didRetrieveAtomEntry:(AtomEntry *)anAtomEntry;
- (void)client:(AtomClient *)aClient didRetrieveAtomFeed:(AtomFeed *)anAtomFeed;
- (void)client:(AtomClient *)aClient didFailWithError:(NSError *)anError;
@end

@interface AtomClient (AtomClientPrivate)
- (NSString *)makeNonce;
- (void)connection:(NSURLConnection *)aConnection didReceiveData:(NSData *)aData;
- (void)connection:(NSURLConnection *)aConnection didFailWithError:(NSError *)anError;
- (void)connectionDidFinishLoading:(NSURLConnection *)aConnection;
- (void)connection:(NSURLConnection *)aConnection didReceiveResponse:(NSURLResponse *)aResponse;
- (NSError *)errorFromResponse:(NSURLResponse *)aResponse data:(NSData *)aData;
- (NSData *)dataFromSoapData:(NSData *)aData;
- (void)sendRequest:(NSURLRequest *)aRequest delegate:(id)aDelegate;
- (NSData *)sendSynchronousRequest:(NSURLRequest *)aRequest returningResponse:(NSURLResponse **)aResponse error:(NSError **)anError;
- (NSURLRequest *)requestWithURL:(NSURL *)aURL method:(NSString *)aMethod object:(id)anObject;

- (NSURLConnection *)connection;
- (void)setConnection:(NSURLConnection *)aConnection;

- (NSMutableData *)data;
- (void)setData:(NSMutableData *)aData;

- (NSURLResponse *)response;
- (void)setResponse:(NSURLResponse *)aResponse;

- (id)delegate;
- (void)setDelegate:(id)aDelegate;

- (int)currentOperation;
- (void)setCurrentOperation:(int)aCurrentOperation;

- (AtomEntry *)entry;
- (void)setEntry:(AtomEntry *)anEntry;
@end
