//
//  AtomContent.m
//  Atom
//
//  Created by Jonathan Younger on Thu Apr 29 2004.
//  Copyright (c) 2004 Daikini Software.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.

//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.

//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#import "AtomContent.h"

@implementation AtomContent
+ (AtomContent *)atomContentWithLocalName:(NSString *)aLocalName
                                  content:(NSString *)aContent
                                     type:(NSString *)aType
                                     mode:(NSString *)aMode
{
    return [[[AtomContent alloc] initWithLocalName:aLocalName
                                           content:aContent
                                              type:aType
                                              mode:aMode] autorelease];
}

- (id)init
{
    return [self initWithLocalName:@"content" content:nil type:nil mode:nil];
}

- (id)initWithLocalName:(NSString *)aLocalName
                content:(NSString *)aContent
                   type:(NSString *)aType
                   mode:(NSString *)aMode
{
    if (self = [super init]) {
        [self setLocalName:aLocalName];
        [self setContent:aContent];
        [self setType:aType];
        [self setMode:aMode];
    }
    
    return self;
}




- (NSString *)type { return type; }

- (void)setType:(NSString *)aType
{
    if (type != aType) {
        [aType retain];
        [type release];
        type = aType;
    }
}


- (NSString *)mode { return mode; }

- (void)setMode:(NSString *)aMode
{
    if (mode != aMode) {
        [aMode retain];
        [mode release];
        mode = aMode;
    }
}


- (NSString *)content { return content; }

- (void)setContent:(NSString *)aContent
{
    if (content != aContent) {
        [aContent retain];
        [content release];
        content = aContent;
    }
}


- (NSString *)stringValue
{
	[[self buffer] setString:@""];
	
    [self writeStartElement];
    
	[self writeAttribute:@"xmlns" value:[[self namespaceUri] absoluteString] required:NO message:nil];
	
    if ((![[self type] isEqualToString:@"text/plain"]) && ([self type] != nil)) {
        [self writeAttribute:@"type" value:[self type] required:NO message:nil];
    }
    
    if (![[[self mode] lowercaseString] isEqualToString:@"xml"]) {
        [self writeAttribute:@"mode" value:[[self mode] lowercaseString] required:NO message:nil];
    }
    
    [self writeAttribute:@"xml:lang" value:[self xmlLang] required:NO message:nil];
    
    [[self buffer] appendString:@">"];
    
    if ([self content] != nil) {
		[[self buffer] appendString:[self content]];
    }
    
    [self writeEndElement];
    

    return [self buffer];
}

- (void)writeStartElement
{
    [[self buffer] appendFormat:@"<%@", [self localName]];
}

- (void)writeEndElement
{
    [[self buffer] appendFormat:@"</%@>", [self localName]];
    [[self buffer] appendString:@"\n"];
}


+ (AtomContent *)atomContentWithLocalName:(NSString *)aLocalName
								 fromTree:(CFXMLTreeRef)aTree
{
	CFXMLNodeRef node; 
	CFDictionaryRef attributeDict; 
	CFXMLElementInfo *elementInfo;
	
	AtomContent *atomContent = [[[AtomContent alloc] init] autorelease];
	[atomContent setLocalName:aLocalName];
	
	node = CFXMLTreeGetNode(aTree); 
	if(CFXMLNodeGetTypeCode(node) == kCFXMLNodeTypeElement) { 
		elementInfo = (CFXMLElementInfo *)CFXMLNodeGetInfoPtr(node); 
		attributeDict = (*elementInfo).attributes;
		
		if (attributeDict != nil) {
			[atomContent setType:[(NSDictionary *)attributeDict objectForKey:@"type"]];
			[atomContent setMode:[(NSDictionary *)attributeDict objectForKey:@"mode"]];
		}
		
		[atomContent setContent:[AtomElement getElementsFromTree:aTree]];
	}

	return atomContent;
}

- (void)dealloc
{
    [type release];
    [mode release];
    [content release];
    
    [super dealloc];
}


@end
