//
//  AtomDate.m
//  Atom
//
//  Created by Jonathan Younger on Thu Apr 29 2004.
//  Copyright (c) 2004 Daikini Software.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.

//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.

//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//




#import "AtomDate.h"


@implementation AtomDate
+ (AtomDate *)atomDateWithLocalName:(NSString *)aLocalName
                               date:(NSDate *)aDate
{
    return [[[AtomDate alloc] initWithLocalName:aLocalName date:aDate] autorelease];
}

- (id)init
{
    return [self initWithLocalName:@"modified" date:[NSDate date]];
}

- (id)initWithLocalName:(NSString *)aLocalName
                   date:(NSDate *)aDate
{
    if (self = [super init]) {
        [self setLocalName:aLocalName];
        if (aDate == nil) {
            [self setDate:[NSDate date]];
        } else {
            [self setDate:aDate];
        }
    }
    
    return self;
}


- (NSDate *)date { return date; }

- (void)setDate:(NSDate *)aDate
{
    if (date != aDate) {
        [aDate retain];
        [date release];
        date = aDate;
    }
}


- (NSString *)stringValue
{
	[[self buffer] setString:@""];
	
    [self writeStartElement];
	
	[self writeAttribute:@"xmlns" value:[[self namespaceUri] absoluteString] required:NO message:nil];
    
    [self writeAttribute:@"xml:lang" value:[self xmlLang] required:NO message:nil];
    
    [[self buffer] appendString:@">"];
    
    [[self buffer] appendString:[[self date] descriptionWithCalendarFormat:@"%Y-%m-%dT%H:%M:%SZ" timeZone:nil locale:nil]];
    
    [self writeEndElement];

    return [self buffer];
}

- (void)writeStartElement
{
    [[self buffer] appendFormat:@"<%@", [self localName]];
}

- (void)writeEndElement
{
    [[self buffer] appendFormat:@"</%@>", [self localName]];
    [[self buffer] appendString:@"\n"];
}

+ (AtomDate *)atomDateWithLocalName:(NSString *)aLocalName
						   fromTree:(CFXMLTreeRef)aTree
{
	NSDate *dateValue;
	AtomDate *atomDate = [[[AtomDate alloc] init] autorelease];
	[atomDate setLocalName:aLocalName];
	dateValue = [NSCalendarDate dateWithString:[self getElementFromTree:aTree] calendarFormat:@"%Y-%m-%dT%H:%M:%SZ"];
	if (dateValue == nil) {
		dateValue = [NSCalendarDate dateWithString:[self getElementFromTree:aTree] calendarFormat:@"%Y-%m-%dT%H:%M:%S-%z"];
	}
	
	[atomDate setDate:dateValue];
	return atomDate;
}

- (void)dealloc
{
    [date release];
    
    [super dealloc];
}
@end
