//
//  AtomElement.m
//  Atom
//
//  Created by Jonathan Younger on Thu Apr 29 2004.
//  Copyright (c) 2004 Daikini Software.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.

//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.

//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#import "AtomElement.h"
#import "AtomExtensions.h"

@implementation AtomElement
- (id)init
{
    if (self = [super init]) {
        buffer = [[NSMutableString alloc] init];
        localName = [[NSString alloc] init];
        xmllang = nil; //@"en-US";
    }
    
    return self;
}

- (NSString *)localName { return localName; }

- (void)setLocalName:(NSString *)aLocalName
{
    if (localName != aLocalName) {
        [aLocalName retain];
        [localName release];
        localName = aLocalName;
    }
}

- (NSString *)namespacePrefix
{
    return @"atom";
}

- (NSURL *)namespaceUri
{
    return [NSURL URLWithString:@"http://purl.org/atom/ns#"];
}

- (NSString *)fullName
{
    return [NSString stringWithFormat:@"%@:%@", [self namespacePrefix], [self localName]];
}

- (NSString *)xmlLang { return xmllang; }
- (void)setXmlLang:(NSString *)anXmlLang
{
    if (xmllang != anXmlLang) {
        [anXmlLang retain];
        [xmllang release];
        xmllang = anXmlLang;
    }
}


- (NSString *)stringValue
{
    return nil;
}

- (NSMutableString *)buffer { return buffer; }


- (void)writeElement:(NSString *)anElementName
               value:(NSString *)aValue
            required:(BOOL)flag
             message:(NSString *)aMessage
{
	
    if(aValue != nil) {
        [[self buffer] appendFormat:@"<%@ xmlns=\"%@\">", anElementName, [[self namespaceUri] absoluteString]];
        [[self buffer] appendString:aValue];
        [[self buffer] appendFormat:@"</%@>", anElementName];
        [[self buffer] appendString:@"\n"];
    }
}

- (void)writeAttribute:(NSString *)anAttributeName
                 value:(NSString *)aValue
              required:(BOOL)flag
               message:(NSString *)aMessage
{
    if (aValue != nil) {
        [[self buffer] appendFormat:@" %@=\"%@\"", anAttributeName, aValue];
    }
}

- (void)writeStartElement
{
    
}

- (void)writeEndElement
{
    
}

+ (NSString *)getElementFromTree:(CFXMLTreeRef)aTree
{
	NSMutableString *elementBuffer = [NSMutableString string];
	int index;
	
	for (index = 0; index < CFTreeGetChildCount(aTree); index++)
	{
		CFXMLNodeRef node = CFXMLTreeGetNode(CFTreeGetChildAtIndex(aTree, index));
		NSString *string = (NSString *)CFXMLNodeGetString(node);
		
		if (string != nil)
		{
			if (CFXMLNodeGetTypeCode(node) == kCFXMLNodeTypeEntityReference)
			{
				if ([string isEqualToString: @"lt"]) {
					string = @"<";
				}
				else if ([string isEqualToString: @"gt"]) {
					string = @">";
				}
				else if ([string isEqualToString: @"quot"]) {
					string = @"\"";
				}
				else if ([string isEqualToString: @"amp"]) {
					string = @"&";
				}
			}
			
			[elementBuffer appendString: string];
		}
	}
	
	return (NSString *)elementBuffer;
}

+ (NSString *)getElementsFromTree:(CFXMLTreeRef)aTree
{
	CFXMLTreeRef childTree;
	int childCount, childIndex;
	NSMutableString *contentBuffer = [NSMutableString string];
	
	childCount = CFTreeGetChildCount(aTree);
	
	for (childIndex = 0; childIndex < childCount; childIndex++) {
		childTree = CFTreeGetChildAtIndex(aTree, childIndex);
		CFDataRef xmlData = CFXMLTreeCreateXMLData(kCFAllocatorDefault, childTree);
		[contentBuffer appendString:[(NSData *)xmlData stringUsingEncoding:NSUTF8StringEncoding]];
		CFRelease(xmlData);
	}
	
	return contentBuffer;
}

+ (CFXMLTreeRef)getTreeFromParent:(CFXMLTreeRef)aParent name:(NSString *)aName
{
	CFXMLTreeRef child;
	NSString *string;
	int index;
	
	if (aParent == nil) { return nil; }
	
	for (index = CFTreeGetChildCount(aParent) - 1; index >= 0; index--) {
		child = CFTreeGetChildAtIndex(aParent, index);
		string = (NSString *)CFXMLNodeGetString(CFXMLTreeGetNode(child));
		
		if ([string isEqualToString: aName]) {
			return child;
		}
	}
	
	return nil;
}


- (void)dealloc
{
    [buffer release];
    [localName release];
    [xmllang release];
	
    [super dealloc];
}
@end
