//
//  AtomEntry.m
//  Atom
//
//  Created by Jonathan Younger on Thu Apr 29 2004.
//  Copyright (c) 2004 Daikini Software.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.

//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.

//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#import "AtomEntry.h"

@implementation AtomEntry
- (id)init
{
    if (self = [super init]) {
        links = [[NSMutableArray alloc] init];
        contributors = [[NSMutableArray alloc] init];
        contents = [[NSMutableArray alloc] init];
        additionalElements = [[NSMutableArray alloc] init];
    }
    
    return self;
}

- (NSString *)localName
{
    return @"entry";
}



- (AtomContent *)title { return title; }

- (void)setTitle:(AtomContent *)aTitle
{
    if (title != aTitle) {
        [aTitle retain];
        [title release];
        title = aTitle;
    }
}


- (NSMutableArray *)links { return links; }

- (void)setLinks:(NSMutableArray *)aLinks
{
    if (links != aLinks) {
        [aLinks retain];
        [links release];
        links = aLinks;
    }
}


- (AtomPerson *)author { return author; }

- (void)setAuthor:(AtomPerson *)anAuthor
{
    if (author != anAuthor) {
        [anAuthor retain];
        [author release];
        author = anAuthor;
    }
}


- (NSMutableArray *)contributors { return contributors; }

- (void)setContributors:(NSMutableArray *)aContributors
{
    if (contributors != aContributors) {
        [aContributors retain];
        [contributors release];
        contributors = aContributors;
    }
}


- (NSURL *)entryId { return entryId; }

- (void)setEntryId:(NSURL *)anEntryId
{
    if (entryId != anEntryId) {
        [anEntryId retain];
        [entryId release];
        entryId = anEntryId;
    }
}


- (AtomDate *)modified { return modified; }

- (void)setModified:(AtomDate *)aModified
{
    if (modified != aModified) {
        [aModified retain];
        [modified release];
        modified = aModified;
    }
}


- (AtomDate *)issued { return issued; }

- (void)setIssued:(AtomDate *)anIssued
{
    if (issued != anIssued) {
        [anIssued retain];
        [issued release];
        issued = anIssued;
    }
}


- (AtomDate *)created { return created; }

- (void)setCreated:(AtomDate *)aCreated
{
    if (created != aCreated) {
        [aCreated retain];
        [created release];
        created = aCreated;
    }
}


- (AtomContent *)summary { return summary; }

- (void)setSummary:(AtomContent *)aSummary
{
    if (summary != aSummary) {
        [aSummary retain];
        [summary release];
        summary = aSummary;
    }
}


- (NSMutableArray *)contents { return contents; }

- (void)setContents:(NSMutableArray *)aContents
{
    if (contents != aContents) {
        [aContents retain];
        [contents release];
        contents = aContents;
    }
}


- (NSMutableArray *)additionalElements { return additionalElements; }

- (void)setAdditionalElements:(NSMutableArray *)anAdditionalElements
{
    if (additionalElements != anAdditionalElements) {
        [anAdditionalElements retain];
        [additionalElements release];
        additionalElements = anAdditionalElements;
    }
}


- (AtomLink *)atomLinkWithRel:(NSString *)aRel type:(NSString *)aType
{
	NSEnumerator *linkEnumerator;
	AtomLink *link;
	
	linkEnumerator = [[self links] objectEnumerator];
	while (link = [linkEnumerator nextObject]) {
		if (([[link rel] isEqualToString:aRel]) && ([[link type] isEqualToString:aType])) {
			return link;
		}
	}
	
	return nil;	
}

- (NSString *)stringValue
{
    NSEnumerator *linkEnumerator;
    AtomLink *link;
    NSEnumerator *contributorEnumerator;
    AtomPerson *contributor;
    NSEnumerator *contentEnumerator;
    AtomContent *content;
    NSEnumerator *scopedElementEnumerator;
    ScopedElement *scopedElement;
    
	[[self buffer] setString:@""];
	
    [[self buffer] appendFormat:@"<%@", [self localName]];
    [self writeAttribute:@"xml:lang" value:[self xmlLang] required:NO message:nil];
	[self writeAttribute:@"xmlns" value:[[self namespaceUri] absoluteString] required:NO message:nil];
    [[self buffer] appendString:@">"];
    [[self buffer] appendString:@"\n"];
    
    if ([self title] != nil) {
        [[self buffer] appendString:[[self title] stringValue]];
    }
    
    linkEnumerator = [[self links] objectEnumerator];
    while (link = [linkEnumerator nextObject]) {
        [[self buffer] appendString:[link stringValue]];
    }
    
    if ([self author] != nil) {
        [[self buffer] appendString:[[self author] stringValue]];
    }
    
    contributorEnumerator = [[self contributors] objectEnumerator];
    while (contributor = [contributorEnumerator nextObject]) {
        [[self buffer] appendString:[contributor stringValue]];
    }
    
    if ([self entryId] != nil) {
        [self writeElement:@"id" value:[[self entryId] absoluteString] required:YES message:@"The id must be specified."];
    }
    
    if ([self modified] != nil) { 
        [[self buffer] appendString:[[self modified] stringValue]];
    }
    
    if ([self issued] != nil) {
        [[self buffer] appendString:[[self issued] stringValue]];
    }
    
    if ([self created] != nil) {
        [[self buffer] appendString:[[self created] stringValue]];
    }
        
    if ([self summary] != nil) {
        [[self buffer] appendString:[[self summary] stringValue]];
    }
    
    contentEnumerator = [[self contents] objectEnumerator];
    while (content = [contentEnumerator nextObject]) {
        [[self buffer] appendString:[content stringValue]];
    }
    
    scopedElementEnumerator = [[self additionalElements] objectEnumerator];
    while (scopedElement = [scopedElementEnumerator nextObject]) {
        [[self buffer] appendString:[scopedElement stringValue]];
    }
    
    [[self buffer] appendFormat:@"</%@>", [self localName]];
    [[self buffer] appendString:@"\n"];

    return [self buffer];
}

+ (AtomEntry *)atomEntryFromString:(NSString *)anXml
{
	return [self atomEntryFromData:[anXml dataUsingEncoding:NSUTF8StringEncoding]];
}

+ (AtomEntry *)atomEntryFromData:(NSData *)aData
{
	CFXMLTreeRef tree = CFXMLTreeCreateFromData(kCFAllocatorDefault, (CFDataRef)aData,
												NULL, kCFXMLParserSkipWhitespace, kCFXMLNodeCurrentVersion);
	CFXMLTreeRef entryTree = [self getTreeFromParent:tree name:@"entry"];
	
	if (entryTree == nil) {
		entryTree = [self getTreeFromParent:[self getTreeFromParent:tree name:@"feed"] name:@"entry"];
	}
	
	AtomEntry *atomEntry = [self atomEntryFromTree:entryTree];
	
	CFRelease(tree);
	
	return atomEntry;
}

+ (AtomEntry *)atomEntryFromTree:(CFXMLTreeRef)aTree
{
	CFXMLTreeRef childTree;
	CFXMLNodeRef childNode;
	int childCount, childIndex;
	NSString *childName;
	
	if (aTree == nil) return nil;
	
	AtomEntry *entry = [[[AtomEntry alloc] init] autorelease];
	
	// Get a count of the top level nodes children.
	childCount = CFTreeGetChildCount(aTree);
	
	for (childIndex = 0; childIndex < childCount; childIndex++) {
		childTree = CFTreeGetChildAtIndex(aTree, childIndex);
		childNode = CFXMLTreeGetNode(childTree);
		childName = (NSString *)CFXMLNodeGetString(childNode);
		
		if ([childName isEqualToString:@"title"]) {
			[entry setTitle:[AtomContent atomContentWithLocalName:childName fromTree:childTree]];
			continue;
		} else if ([childName isEqualToString:@"link"]) {
			[[entry links] addObject:[AtomLink atomLinkFromTree:childTree]];
			continue;
		} else if ([childName isEqualToString:@"id"]) {
			[entry setEntryId:[NSURL URLWithString:[self getElementFromTree:childTree]]];
			continue;
		} else if ([childName isEqualToString:@"issued"]) {
			[entry setIssued:[AtomDate atomDateWithLocalName:childName fromTree:childTree]];
			continue;
		} else if ([childName isEqualToString:@"created"]) {
			[entry setCreated:[AtomDate atomDateWithLocalName:childName fromTree:childTree]];
			continue;
		} else if ([childName isEqualToString:@"modified"]) {
			[entry setModified:[AtomDate atomDateWithLocalName:childName fromTree:childTree]];
			continue;
		} else if ([childName isEqualToString:@"summary"]) {
			[entry setSummary:[AtomContent atomContentWithLocalName:childName fromTree:childTree]];
			continue;
		} else if ([childName isEqualToString:@"author"]) {
			[entry setAuthor:[AtomPerson atomPersonWithLocalName:childName fromTree:childTree]];
			continue;
		} else if ([childName isEqualToString:@"contributor"]) {
			[[entry contributors] addObject:[AtomPerson atomPersonWithLocalName:childName fromTree:childTree]];
			continue;
		} else if ([childName isEqualToString:@"content"]) {
			[[entry contents] addObject:[AtomContent atomContentWithLocalName:childName fromTree:childTree]];
			continue;
		} else if ([childName isEqualToString:@"dc:subject"]) {
			[[entry contents] addObject:[DcElement dcElementWithLocalName:childName fromTree:childTree]];
			continue;
		}
		
		//NSLog(@"Child = %@", childName);
	}
		
	
	return entry;	
}

- (void)dealloc
{
    [title release];
    [links release];
    [author release];
    [contributors release];
    [entryId release];
    [modified release];
    [issued release];
    [created release];
    [summary release];
    [contents release];
    [additionalElements release];
    
    [super dealloc];
}
@end
