//
//  AtomFeed.h
//  Atom
//
//  Created by Jonathan Younger on Thu Apr 29 2004.
//  Copyright (c) 2004 Daikini Software.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.

//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.

//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#import <Foundation/Foundation.h>
#import "AtomElement.h"
#import "AtomEntry.h"
#import "AtomContent.h"
#import "AtomLink.h"
#import "AtomGenerator.h"
#import "AtomPerson.h"
#import "AtomDate.h"
#import "ScopedElement.h"

/*! @class AtomFeed
	@brief An Atom Feed construct.
*/

@interface AtomFeed : AtomElement {
    AtomContent *title;
    NSMutableArray *links;
    AtomPerson *author;
    NSMutableArray *contributors;
    AtomContent *tagline;
    NSURL *feedId;
    AtomGenerator *generator;
    AtomContent *copyright;
    AtomContent *info;
    AtomDate *modified;
    NSMutableArray *entries;
    NSMutableArray *additionalElements;
}

+ (AtomFeed *)atomFeedFromString:(NSString *)anXml;
+ (AtomFeed *)atomFeedFromData:(NSData *)aData;
+ (AtomFeed *)atomFeedFromTree:(CFXMLTreeRef)aTree;

- (AtomLink *)atomLinkWithTitle:(NSString *)aTitle rel:(NSString *)aRel type:(NSString *)aType;

- (AtomContent *)title;
- (void)setTitle:(AtomContent *)aTitle;

- (NSMutableArray *)links;
- (void)setLinks:(NSMutableArray *)aLinks;

- (AtomPerson *)author;
- (void)setAuthor:(AtomPerson *)anAuthor;

- (NSMutableArray *)contributors;
- (void)setContributors:(NSMutableArray *)aContributors;

- (AtomContent *)tagline;
- (void)setTagline:(AtomContent *)aTagline;

- (NSURL *)feedId;
- (void)setFeedId:(NSURL *)aFeedId;

- (AtomGenerator *)generator;
- (void)setGenerator:(AtomGenerator *)aGenerator;

- (AtomContent *)copyright;
- (void)setCopyright:(AtomContent *)aCopyright;

- (AtomContent *)info;
- (void)setInfo:(AtomContent *)anInfo;

- (AtomDate *)modified;
- (void)setModified:(AtomDate *)aModified;

- (NSMutableArray *)entries;
- (void)setEntries:(NSMutableArray *)anEntries;

- (NSMutableArray *)additionalElements;
- (void)setAdditionalElements:(NSMutableArray *)anAdditionalElements;

- (NSString *)version;

@end
