//
//  AtomFeed.m
//  Atom
//
//  Created by Jonathan Younger on Thu Apr 29 2004.
//  Copyright (c) 2004 Daikini Software.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.

//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.

//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#import "AtomFeed.h"


@implementation AtomFeed
- (id)init
{
    if (self = [super init]) {
        links = [[NSMutableArray alloc] init];
        contributors = [[NSMutableArray alloc] init];
        entries = [[NSMutableArray alloc] init];
        additionalElements = [[NSMutableArray alloc] init];
    }
    
    return self;
}



- (AtomContent *)title { return title; }

- (void)setTitle:(AtomContent *)aTitle
{
    if (title != aTitle) {
        [aTitle retain];
        [title release];
        title = aTitle;
    }
}


- (NSMutableArray *)links { return links; }

- (void)setLinks:(NSMutableArray *)aLinks
{
    if (links != aLinks) {
        [aLinks retain];
        [links release];
        links = aLinks;
    }
}


- (AtomPerson *)author { return author; }

- (void)setAuthor:(AtomPerson *)anAuthor
{
    if (author != anAuthor) {
        [anAuthor retain];
        [author release];
        author = anAuthor;
    }
}


- (NSMutableArray *)contributors { return contributors; }

- (void)setContributors:(NSMutableArray *)aContributors
{
    if (contributors != aContributors) {
        [aContributors retain];
        [contributors release];
        contributors = aContributors;
    }
}


- (AtomContent *)tagline { return tagline; }

- (void)setTagline:(AtomContent *)aTagline
{
    if (tagline != aTagline) {
        [aTagline retain];
        [tagline release];
        tagline = aTagline;
    }
}


- (NSURL *)feedId { return feedId; }

- (void)setFeedId:(NSURL *)aFeedId
{
    if (feedId != aFeedId) {
        [aFeedId retain];
        [feedId release];
        feedId = aFeedId;
    }
}


- (AtomGenerator *)generator { return generator; }

- (void)setGenerator:(AtomGenerator *)aGenerator
{
    if (generator != aGenerator) {
        [aGenerator retain];
        [generator release];
        generator = aGenerator;
    }
}


- (AtomContent *)copyright { return copyright; }

- (void)setCopyright:(AtomContent *)aCopyright
{
    if (copyright != aCopyright) {
        [aCopyright retain];
        [copyright release];
        copyright = aCopyright;
    }
}


- (AtomContent *)info { return info; }

- (void)setInfo:(AtomContent *)anInfo
{
    if (info != anInfo) {
        [anInfo retain];
        [info release];
        info = anInfo;
    }
}


- (AtomDate *)modified { return modified; }

- (void)setModified:(AtomDate *)aModified
{
    if (modified != aModified) {
        [aModified retain];
        [modified release];
        modified = aModified;
    }
}


- (NSMutableArray *)entries { return entries; }

- (void)setEntries:(NSMutableArray *)anEntries
{
    if (entries != anEntries) {
        [anEntries retain];
        [entries release];
        entries = anEntries;
    }
}


- (NSMutableArray *)additionalElements { return additionalElements; }

- (void)setAdditionalElements:(NSMutableArray *)anAdditionalElements
{
    if (additionalElements != anAdditionalElements) {
        [anAdditionalElements retain];
        [additionalElements release];
        additionalElements = anAdditionalElements;
    }
}


- (NSString *)version
{
    return @"0.3";
}

- (NSString *)localName
{
    return @"feed";
}

- (NSString *)stringValue
{
    NSEnumerator *linkEnumerator;
    AtomLink *link;
    NSEnumerator *contributorEnumerator;
    AtomPerson *contributor;
    NSEnumerator *entryEnumerator;
    AtomEntry *entry;
    NSEnumerator *scopedElementEnumerator;
    ScopedElement *scopedElement;
    
	[[self buffer] setString:@""];
	
    [[self buffer] appendFormat:@"<%@", [self localName]];
    
    [self writeAttribute:@"version" value:[self version] required:NO message:nil];
    [self writeAttribute:@"xml:lang" value:[self xmlLang] required:NO message:nil];
    [self writeAttribute:@"xmlns" value:[[self namespaceUri] absoluteString] required:NO message:nil];
    
    [[self buffer] appendString:@">"];
    [[self buffer] appendString:@"\n"];
    
    if ([self title] != nil) {
        [[self buffer] appendString:[[self title] stringValue]];
    }
    
    linkEnumerator = [[self links] objectEnumerator];
    while (link = [linkEnumerator nextObject]) {
        [[self buffer] appendString:[link stringValue]];
    }
    
    if ([self author] != nil) {
        [[self buffer] appendString:[[self author] stringValue]];
    }
    
    contributorEnumerator = [[self contributors] objectEnumerator];
    while(contributor = [contributorEnumerator nextObject]) {
        [[self buffer] appendString:[contributor stringValue]];
    }
    
    if ([self tagline] != nil) {
        [[self buffer] appendString:[[self tagline] stringValue]];
    }
    
    [self writeElement:@"id" value:[[self feedId] absoluteString] required:NO message:nil];
    
    if ([self generator] != nil) {
        [[self buffer] appendString:[[self generator] stringValue]];
    }
    
    if ([self copyright] != nil) {
        [[self buffer] appendString:[[self copyright] stringValue]];
    }
    
    if ([self info] != nil) {
        [[self buffer] appendString:[[self info] stringValue]];
    }
    
    if ([self modified] != nil) {
        [[self buffer] appendString:[[self modified] stringValue]];
    }
    
    
    scopedElementEnumerator = [[self additionalElements] objectEnumerator];
    while (scopedElement = [scopedElementEnumerator nextObject]) {
        [[self buffer] appendString:[scopedElement stringValue]];
    }
    
    entryEnumerator = [[self entries] objectEnumerator];
    while (entry = [entryEnumerator nextObject]) {
        [[self buffer] appendString:[entry stringValue]];
    }
    
    [[self buffer] appendFormat:@"</%@>", [self localName]];
    [[self buffer] appendString:@"\n"];

    
    return [self buffer];
}

+ (AtomFeed *)atomFeedFromString:(NSString *)anXml
{
	return [self atomFeedFromData:[anXml dataUsingEncoding:NSUTF8StringEncoding]];
}

+ (AtomFeed *)atomFeedFromData:(NSData *)aData
{
	CFXMLTreeRef tree = CFXMLTreeCreateFromData(kCFAllocatorDefault, (CFDataRef)aData,
												NULL, kCFXMLParserSkipWhitespace, kCFXMLNodeCurrentVersion);
	AtomFeed *feed = [self atomFeedFromTree:[self getTreeFromParent:tree name:@"feed"]];
	
	CFRelease(tree);
	
	return feed;
}

+ (AtomFeed *)atomFeedFromTree:(CFXMLTreeRef)aTree
{
	CFXMLTreeRef childTree;
	CFXMLNodeRef childNode;
	int childCount, childIndex;
	NSString *childName;
	
	if (aTree == nil) return nil;
	
	AtomFeed *feed = [[[AtomFeed alloc] init] autorelease];
	
	// Get a count of the top level nodes children.
	childCount = CFTreeGetChildCount(aTree);
	
	for (childIndex = 0; childIndex < childCount; childIndex++) {
		childTree = CFTreeGetChildAtIndex(aTree, childIndex);
		childNode = CFXMLTreeGetNode(childTree);
		childName = (NSString *)CFXMLNodeGetString(childNode);
		
		if ([childName isEqualToString:@"title"]) {
			[feed setTitle:[AtomContent atomContentWithLocalName:childName fromTree:childTree]];
			continue;
		} else if ([childName isEqualToString:@"link"]) {
			[[feed links] addObject:[AtomLink atomLinkFromTree:childTree]];
			continue;
		} else if ([childName isEqualToString:@"id"]) {
			[feed setFeedId:[NSURL URLWithString:[self getElementFromTree:childTree]]];
			continue;
		} else if ([childName isEqualToString:@"modified"]) {
			[feed setModified:[AtomDate atomDateWithLocalName:childName fromTree:childTree]];
			continue;
		} else if ([childName isEqualToString:@"info"]) {
			[feed setInfo:[AtomContent atomContentWithLocalName:childName fromTree:childTree]];
			continue;
		} else if ([childName isEqualToString:@"tagline"]) {
			[feed setTagline:[AtomContent atomContentWithLocalName:childName fromTree:childTree]];
			continue;
		} else if ([childName isEqualToString:@"copyright"]) {
			[feed setCopyright:[AtomContent atomContentWithLocalName:childName fromTree:childTree]];
			continue;
		} else if ([childName isEqualToString:@"author"]) {
			[feed setAuthor:[AtomPerson atomPersonWithLocalName:childName fromTree:childTree]];
			continue;
		} else if ([childName isEqualToString:@"contributor"]) {
			[[feed contributors] addObject:[AtomPerson atomPersonWithLocalName:childName fromTree:childTree]];
			continue;
		} else if ([childName isEqualToString:@"generator"]) {
			[feed setGenerator:[AtomGenerator atomGeneratorFromTree:childTree]];
			continue;
		} else if ([childName isEqualToString:@"entry"]) {
			[[feed entries] addObject:[AtomEntry atomEntryFromTree:childTree]];
			continue;
		}
		
		//NSLog(@"Child = %@", childName);
	}
	
	
	return feed;	
}

- (AtomLink *)atomLinkWithTitle:(NSString *)aTitle rel:(NSString *)aRel type:(NSString *)aType
{
	NSEnumerator *linkEnumerator;
	AtomLink *link;
	
	linkEnumerator = [[self links] objectEnumerator];
	while (link = [linkEnumerator nextObject]) {
		if (([[link title] isEqualToString:aTitle]) && ([[link rel] isEqualToString:aRel]) && ([[link type] isEqualToString:aType])) {
			return link;
		}
	}
	
	return nil;
}

- (void)dealloc
{
    [title release];
    [links release];
    [author release];
    [contributors release];
    [tagline release];
    [feedId release];
    [generator release];
    [copyright release];
    [info release];
    [modified release];
    [entries release];
    [additionalElements release];
    
    [super dealloc];
}
@end
