//
//  AtomGenerator.m
//  Atom
//
//  Created by Jonathan Younger on Thu Apr 29 2004.
//  Copyright (c) 2004 Daikini Software.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.

//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.

//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#import "AtomGenerator.h"


@implementation AtomGenerator
+ (AtomGenerator *)atomGenerator
{
    return [[[AtomGenerator alloc] init] autorelease];
}

- (id)init
{
    if (self = [super init]) {
        [self setGenerator:@"Generated by Atom.framework - http://www.daikini.com/source/atom.framework/"];
        [self setURL:[NSURL URLWithString:@"http://www.daikini.com/source/atom.framework/"]];
        [self setVersion:@"0.5"];
    }
    
    return self;
}




- (NSString *)generator { return generator; }

- (void)setGenerator:(NSString *)aGenerator
{
    if (generator != aGenerator) {
        [aGenerator retain];
        [generator release];
        generator = aGenerator;
    }
}


- (NSURL *)URL { return URL; }

- (void)setURL:(NSURL *)aURL
{
    if (URL != aURL) {
        [aURL retain];
        [URL release];
        URL = aURL;
    }
}


- (NSString *)version { return version; }

- (void)setVersion:(NSString *)aVersion
{
    if (version != aVersion) {
        [aVersion retain];
        [version release];
        version = aVersion;
    }
}


- (NSString *)stringValue
{
	[[self buffer] setString:@""];
    [[self buffer] appendString:@"<generator"];
	[self writeAttribute:@"xmlns" value:[[self namespaceUri] absoluteString] required:NO message:nil];
    [self writeAttribute:@"xml:lang" value:[self xmlLang] required:NO message:nil];
    [self writeAttribute:@"url" value:[[self URL] absoluteString] required:NO message:nil];
    [self writeAttribute:@"version" value:[self version] required:NO message:nil];
    [[self buffer] appendString:@">"];
    [[self buffer] appendString:[self generator]];
    [[self buffer] appendString:@"</generator>"];
    [[self buffer] appendString:@"\n"];
    
    return [self buffer];
}

+ (AtomGenerator *)atomGeneratorFromTree:(CFXMLTreeRef)aTree
{
	CFXMLNodeRef node; 
	CFDictionaryRef attributeDict; 
	CFXMLElementInfo *elementInfo; 
	
	AtomGenerator *atomGenerator = [[[AtomGenerator alloc] init] autorelease];
	
	node = CFXMLTreeGetNode(aTree); 
	if(CFXMLNodeGetTypeCode(node) == kCFXMLNodeTypeElement) { 
		elementInfo = (CFXMLElementInfo *)CFXMLNodeGetInfoPtr(node); 
		attributeDict = (*elementInfo).attributes;
		[atomGenerator setURL:[NSURL URLWithString:[(NSDictionary *)attributeDict objectForKey:@"url"]]];
		[atomGenerator setVersion:[(NSDictionary *)attributeDict objectForKey:@"version"]];
		[atomGenerator setGenerator:[self getElementFromTree:aTree]];
	}
	
	return atomGenerator;
}

- (void)dealloc
{
    [generator release];
    [URL release];
    [version release];
    
    [super dealloc];
}
@end
