//
//  AtomLink.m
//  Atom
//
//  Created by Jonathan Younger on Thu Apr 29 2004.
//  Copyright (c) 2004 Daikini Software.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.

//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.

//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#import "AtomLink.h"


@implementation AtomLink
+ (AtomLink *)atomLinkWithRel:(NSString *)aRel
                         type:(NSString *)aType
                         HREF:(NSURL *)anHREF
                        title:(NSString *)aTitle
{
    return [[[AtomLink alloc] initWithRel:aRel 
                                     type:aType
                                     HREF:anHREF
                                    title:aTitle] autorelease];
    
}

-(id)initWithRel:(NSString *)aRel
            type:(NSString *)aType 
            HREF:(NSURL *)anHREF 
           title:(NSString *)aTitle
{
    if (self = [super init]) {
        [self setRel:aRel];
        [self setType:aType];
        [self setHREF:anHREF];
        [self setTitle:aTitle];
    }
    
    return self;
}

- (NSString *)localName
{
    return @"link";
}



- (NSString *)rel { return rel; }

- (void)setRel:(NSString *)aRel
{
    if (rel != aRel) {
        [aRel retain];
        [rel release];
        rel = aRel;
    }
}


- (NSString *)type { return type; }

- (void)setType:(NSString *)aType
{
    if (type != aType) {
        [aType retain];
        [type release];
        type = aType;
    }
}


- (NSURL *)HREF { return HREF; }

- (void)setHREF:(NSURL *)anHREF
{
    if (HREF != anHREF) {
        [anHREF retain];
        [HREF release];
        HREF = anHREF;
    }
}


- (NSString *)title { return title; }

- (void)setTitle:(NSString *)aTitle
{
    if (title != aTitle) {
        [aTitle retain];
        [title release];
        title = aTitle;
    }
}


- (NSString *)stringValue
{
	[[self buffer] setString:@""];
    [self writeStartElement];
	[self writeAttribute:@"xmlns" value:[[self namespaceUri] absoluteString] required:NO message:nil];
    [self writeAttribute:@"xml:lang" value:[self xmlLang] required:NO message:nil];
    [self writeAttribute:@"rel" value:[self rel] required:NO message:nil];
    [self writeAttribute:@"type" value:[self type] required:NO message:nil];
    [self writeAttribute:@"href" value:[[self HREF] absoluteString] required:NO message:nil];
    [self writeAttribute:@"title" value:[self title] required:NO message:nil];    
    [self writeEndElement];

    return [self buffer];
}

- (void)writeStartElement
{
    [[self buffer] appendFormat:@"<%@", [self localName]];
}

- (void)writeEndElement
{
    [[self buffer] appendString:@" />"];
    [[self buffer] appendString:@"\n"];
}

+ (AtomLink *)atomLinkFromTree:(CFXMLTreeRef)aTree
{
	CFXMLNodeRef node; 
	CFDictionaryRef attributeDict; 
	CFXMLElementInfo *elementInfo; 
	
	AtomLink *link = [[[AtomLink alloc] init] autorelease];
	
	node = CFXMLTreeGetNode(aTree); 
	if (CFXMLNodeGetTypeCode(node) == kCFXMLNodeTypeElement) { 
		elementInfo = (CFXMLElementInfo *)CFXMLNodeGetInfoPtr(node); 
		attributeDict = (*elementInfo).attributes;
		[link setRel:[(NSDictionary *)attributeDict objectForKey:@"rel"]];
		[link setType:[(NSDictionary *)attributeDict objectForKey:@"type"]];
		[link setHREF:[NSURL URLWithString:[(NSDictionary *)attributeDict objectForKey:@"href"]]];
		[link setTitle:[(NSDictionary *)attributeDict objectForKey:@"title"]];
	}

	return link;
}

- (void)dealloc
{
    [rel release];
    [type release];
    [HREF release];
    [title release];
    
    [super dealloc];
}

@end
