//
//  AtomPerson.m
//  Atom
//
//  Created by Jonathan Younger on Thu Apr 29 2004.
//  Copyright (c) 2004 Daikini Software.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.

//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.

//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#import "AtomPerson.h"


@implementation AtomPerson
+ (AtomPerson *)atomPersonWithLocalName:(NSString *)aLocalName
                                   name:(NSString *)aName
                                    URL:(NSURL *)aURL
                                  email:(NSString *)anEmail
{
    return [[[AtomPerson alloc] initWithLocalName:aLocalName
                                             name:aName
                                              URL:aURL
                                            email:anEmail] autorelease];
}

- (id)init
{
    return [self initWithLocalName:@"author" name:nil URL:nil email:nil];
}

- (id)initWithLocalName:(NSString *)aLocalName 
                   name:(NSString *)aName
                    URL:(NSURL *)aURL
                  email:(NSString *)anEmail
{
    if (self = [super init]) {
        [self setLocalName:aLocalName];
        [self setName:aName];
        [self setURL:aURL];
        [self setEmail:anEmail];
    }
    
    return self;
    
}



- (NSString *)name { return name; }

- (void)setName:(NSString *)aName
{
    if (name != aName) {
        [aName retain];
        [name release];
        name = aName;
    }
}


- (NSURL *)URL { return URL; }

- (void)setURL:(NSURL *)aURL
{
    if (URL != aURL) {
        [aURL retain];
        [URL release];
        URL = aURL;
    }
}


- (NSString *)email { return email; }

- (void)setEmail:(NSString *)anEmail
{
    if (email != anEmail) {
        [anEmail retain];
        [email release];
        email = anEmail;
    }
}

- (NSString *)stringValue
{
	[[self buffer] setString:@""];
    [self writeStartElement];
	[self writeAttribute:@"xmlns" value:[[self namespaceUri] absoluteString] required:NO message:nil];
    [self writeAttribute:@"xml:lang" value:[self xmlLang] required:NO message:nil];
    [[self buffer] appendString:@">"];
    [[self buffer] appendString:@"\n"];
    [self writeElement:@"name" value:[self name] required:YES message:@"The name of the author must be specified."];
    [self writeElement:@"url" value:[[self URL] absoluteString] required:NO message:nil];
    [self writeElement:@"email" value:[self email] required:NO message:nil];
    [self writeEndElement];
    
    return [self buffer];
}

- (void)writeStartElement
{
    [[self buffer] appendFormat:@"<%@", [self localName]];
}

- (void)writeEndElement
{
    [[self buffer] appendFormat:@"</%@>", [self localName]];
    [[self buffer] appendString:@"\n"];
}

+ (AtomPerson *)atomPersonWithLocalName:(NSString *)aLocalName
							   fromTree:(CFXMLTreeRef)aTree
{
	CFXMLTreeRef childTree;
	CFXMLNodeRef childNode;
	int childCount, childIndex;
	NSString *childName;
	
	AtomPerson *atomPerson = [[[AtomPerson alloc] init] autorelease];
	[atomPerson setLocalName:aLocalName];
	
	childCount = CFTreeGetChildCount(aTree);
	
	for (childIndex = 0; childIndex < childCount; childIndex++) {
		childTree = CFTreeGetChildAtIndex(aTree,childIndex);
		childNode = CFXMLTreeGetNode(childTree);
		childName = (NSString *)CFXMLNodeGetString(childNode);
		
		if ([childName isEqualToString:@"name"]) {
			[atomPerson setName:[self getElementFromTree:childTree]];
			continue;
		} else if ([childName isEqualToString:@"url"]) {
			[atomPerson setURL:[NSURL URLWithString:[self getElementFromTree:childTree]]];
			continue;
		} else if ([childName isEqualToString:@"email"]) {
			[atomPerson setEmail:[self getElementFromTree:childTree]];
			continue;
		}
	}
	
	return atomPerson;
	
}

- (void)dealloc
{
    [name release];
    [URL release];
    [email release];
    
    [super dealloc];
}
@end
