//
//  Plugin.m
//

#import "Plugin.h"

@implementation Plugin
- (id) initWithWebView:(WebView *)webview
{
    self = [super init];
    
    _webview = webview;

    return self;
}

- (void) windowScriptObjectAvailable:(WebScriptObject *)wso
{
    [wso setValue:self forKey:@"ExecPlugin"];
}

+ (NSString *) webScriptNameForSelector:(SEL)s
{
    if (s == @selector(exec:))
        return @"exec";
    else
        return nil;
}

+ (BOOL) isSelectorExcludedFromWebScript:(SEL)s
{
    if (s == @selector(exec:))
        return NO;
    return YES;
}

- (void) exec:(NSString *)str
{
    NSURL *url;
    bool useAtom = false;

    if (!str || [str length] < 8)
    {
        [_webview stringByEvaluatingJavaScriptFromString:@"set_content('<div id=\"info\">Error loading RSS-feed</div>');"];
        return;
    }
        // allow feed: instead of http: like Safari does
        if ([[str substringToIndex:5] isEqualToString:@"feed:"])
            url = [NSURL URLWithString:[NSString stringWithFormat:@"http:%@", [str substringFromIndex:5]]];
        else
            url = [NSURL URLWithString:str];

    NS_DURING
        _rssFeed = [[RSS alloc] initWithURL:url normalize:YES];
    NS_HANDLER
        // Maybe it's an Atom feed
        NSError *atomError;
        _atomClient = [[AtomClient alloc] initWithUserName:@"user name" password:@"password"];
        _atomFeed = [_atomClient retrieveAtomFeedWithContentsOfURL:url returningError:&atomError];

        if (_atomFeed != nil) {
            // Okay, try to parse the Atom feed now...
            useAtom = true;
        }
        else
            NSLog(@"Error: [%d] - %@", [atomError code], [atomError localizedDescription]);

        if (!useAtom)
        {
            NSLog(@"Exception: %@ %@", [localException name], [localException reason]);
            [_webview stringByEvaluatingJavaScriptFromString:[NSString stringWithFormat:@"set_content('<div id=\"info\"><b>Error loading RSS-feed: %@</b><br />&nbsp;<br />%@</div>');", [localException name], [localException reason]]];
            return;
        }
    NS_ENDHANDLER

    _content = [[NSMutableString alloc] init];
    
    NSString *link = nil, *title = nil, *description = nil;
    
    if (useAtom)
    {
        title = [[_atomFeed title] content];
        NSMutableArray *a = [_atomFeed links];
        
        description = [[_atomFeed tagline] content];
        
        NSEnumerator *enumerator = [a objectEnumerator];
        AtomLink *item;
        
        if (item = [enumerator nextObject])
        {
            //NSLog(@"%@ %@", [item rel], [item title]);
            link = [[item HREF] absoluteString];
        }
    }
    else
    {
        link = [[_rssFeed headerItems] objectForKey:@"link"];
        title = [[_rssFeed headerItems] objectForKey:@"title"];
        description = [[_rssFeed headerItems] objectForKey:@"description"];
    }
    
    [_content appendString:@"<img id=\"reload\" src=\"Reload.png\" onclick=\"update_rss();\" />"];

    [_content appendString:@"<h1>"];
    if (link)
            [_content appendFormat:@"<a href=\"javascript:widget.openURL('%@');\">", link];
    if (title)
        [_content appendString:title];
    if (link)
        [_content appendString:@"</a>"];
    [_content appendString:@"</h1>"];
    
    if (description)
        [_content appendFormat:@"<p id=\"description\">%@</p>", description];

	NSEnumerator *enumerator;
    if (useAtom)
    {
        enumerator = [[_atomFeed entries] objectEnumerator];
        //NSLog(@"entries: %@", [[_atomFeed entries] description]);
    }
    else
    {
        enumerator = [[_rssFeed newsItems] objectEnumerator];
    }

    id item;
        
    while (item = [enumerator nextObject])
    {
        if (useAtom)
        {
            AtomEntry *e = item;
            AtomLink *l;

            NSEnumerator *en;
            en = [[e links] objectEnumerator];
            if (l = [en nextObject])
                    link = [[l HREF] absoluteString];
            
            title = [[item title] content];
            
            description = [[e summary] content];

            //en = [[e summary] objectEnumerator];
            //if (l = [en nextObject])
            //        link = [[l HREF] absoluteString];
            //NSLog(@"%@", [[en nextObject] description]);
            
            //description = @"";
        }
        else
        {
            link = [item objectForKey:@"link"];
            title = [item objectForKey:@"title"];
            description = [item objectForKey:@"description"];
        }

        [_content appendString:@"<div class=\"entry\" onclick=\""];
        if (link)
            [_content appendFormat:@"widget.openURL('%@');return false;", link];
        [_content appendString:@"\"><h3>"];
        if (title)
            [_content appendString:title];
        [_content appendString:@"</h3>"];
        if (description)
            [_content appendFormat:@"<p>%@</p>", description];
        [_content appendString:@"</div>"];
    }

    // escape the content string
    [_content replaceOccurrencesOfString:@"\\" withString:@"\\\\" options:0 range:NSMakeRange(0, [_content length])];
    [_content replaceOccurrencesOfString:@"'" withString:@"\\'" options:0 range:NSMakeRange(0, [_content length])];
    //[_content replaceOccurrencesOfString:@"\"" withString:@"\\\"" options:0 range:NSMakeRange(0, [_content length])];
    [_content replaceOccurrencesOfString:@"\n" withString:@" " options:0 range:NSMakeRange(0, [_content length])];
    [_content replaceOccurrencesOfString:@"\r" withString:@"" options:0 range:NSMakeRange(0, [_content length])];

     //NSLog(@"%@", _content);
    
    [_webview stringByEvaluatingJavaScriptFromString:[NSString stringWithFormat:@"set_content('%@');", _content]];
    
    [_content release];

    if (useAtom)
        [_atomClient release];
    else
        [_rssFeed release];

}
@end
